#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>

#include "leakage.h"



#define LK_PIN_CTRL             3
#define LK_PIN_DATA             2



static uint8_t lk_state;
static uint16_t lk_val;
volatile static uint8_t lk_leak_thr;



static uint16_t lk_measure() {
    uint16_t val;

    // start capacitor charging
    PORTD |= _BV(LK_PIN_CTRL);

    // wait for measure finish
    for (val = 0; val < LK_MEASURE_THRESHOLD; val++) {
        if ((PIND & _BV(LK_PIN_DATA)) != 0) break;
    }

    // to uncharge capacitor
    PORTD &= ~_BV(LK_PIN_CTRL);

    return val;
}


//
// interface functions
//

void lk_init() {
    DDRD |= _BV(LK_PIN_CTRL);
    PORTD &= ~_BV(LK_PIN_CTRL);

    lk_state = LK_STATUS_ERROR;
    lk_leak_thr = 0;
}


uint8_t lk_get_state() {
    return lk_state;
}


uint16_t lk_get_val() {
    return lk_val;
}


void lk_poll() {
    if (lk_state != LK_STATUS_LEAK) {
        lk_val = lk_measure();

        if (lk_val < LK_SENSOR_THRESHOLD) {
            lk_state = LK_STATUS_ERROR;
            lk_leak_thr = 0;
        } else if (lk_val < LK_LEAKAGE_THRESHOLD) {
          lk_state = LK_STATUS_OK;
          lk_leak_thr = 0;
        } else {
            if (lk_leak_thr++ >= 1) {
                lk_state = LK_STATUS_LEAK;
            }
        }
    }
}
