#include <avr/io.h>

#include "led.h"



#define ARRAY_SIZE(a)       (sizeof(a) / sizeof((a)[0]))



#define LED_PORT_A          0
#define LED_PORT_B          1
#define LED_PORT_C          2
#define LED_PORT_D          3


#define LED_FLASH_PERIOD    15  // 0.15s * 1000 / 10ucs
#define LED_BLINK_PERIOD    20  // 0.50s * 1000 / 10ucs



// LEDs state and configuration

typedef struct {
    uint8_t port;
    uint8_t pin;
    uint8_t value;
    uint8_t blink;
} led_state_t;


static led_state_t led_state[] = {
    [LED_VALVE_CLOSE]   = { .port = LED_PORT_B, .pin = _BV(5) },
    [LED_VALVE_OPEN]    = { .port = LED_PORT_B, .pin = _BV(4) },
    [LED_VALVE_MANUAL]  = { .port = LED_PORT_B, .pin = _BV(3) },

//    [LED_TX]            = { .port = LED_PORT_C, .pin = _BV(3) },
    [LED_RX]            = { .port = LED_PORT_C, .pin = _BV(2) },

    [LED_PWR_LINE]      = { .port = LED_PORT_C, .pin = _BV(4) },
    [LED_PWR_BAT]       = { .port = LED_PORT_C, .pin = _BV(5) },

    [LED_LEAKAGE]       = { .port = LED_PORT_C, .pin = _BV(0) }
};


static uint8_t led_flash_cnt;
static uint8_t led_flash_state;


// set LED on/off
static void led_on_off(uint8_t id, uint8_t value) {
    switch(led_state[id].port) {
    case LED_PORT_B:
        if (value == 0)
            PORTB &= ~led_state[id].pin;
        else
            PORTB |= led_state[id].pin;
        break;
    case LED_PORT_C:
        if (value == 0)
            PORTC &= ~led_state[id].pin;
        else
            PORTC |= led_state[id].pin;
        break;
//    case LED_PORT_D:
//        if (value == 0)
//            PORTD &= ~led_state[id].pin;
//        else
//            PORTD |= led_state[id].pin;
//        break;
    }
}


//
// interface functions
//

void led_init() {
    uint8_t id;

    led_flash_state = 0;
    led_flash_cnt = 0;

    for (id = 0; id < ARRAY_SIZE(led_state); id++) {
        switch(led_state[id].port) {
        case LED_PORT_B:
            DDRB |= led_state[id].pin;
            break;
        case LED_PORT_C:
            DDRC |= led_state[id].pin;
            break;
//        case LED_PORT_D:
//            DDRD |= led_state[id].pin;
//            break;
        }
    }
}


void led_set(uint8_t id, uint8_t value) {
    led_state[id].value = value;
    if (value == LED_ON) {
        led_on_off(id, 1);
    } else if (value == LED_OFF) {
        led_on_off(id, 0);
    } else if (value == LED_BLINK) {
        led_on_off(id, 1);
        led_state[id].blink = 0;
    }
}


void led_poll() {
    uint8_t id;
    uint8_t flash_update;

    // update flash state
    if (led_flash_cnt++ >= LED_FLASH_PERIOD) {
        led_flash_cnt = 0;
        led_flash_state = ~led_flash_state;
        flash_update = 1;
    } else {
        flash_update = 0;
    }

    for (id = 0; id < ARRAY_SIZE(led_state); id++) {
        if (led_state[id].value == LED_FLASH && flash_update) {
            led_on_off(id, led_flash_state);
        } else if (led_state[id].value == LED_FLASH_INV && flash_update) {
            led_on_off(id, ~led_flash_state);
        } else if (led_state[id].value == LED_BLINK) {
            if (led_state[id].blink++ >= LED_BLINK_PERIOD) {
                led_on_off(id, 0);
                led_state[id].value = LED_OFF;
            }
        }
    }
}
