#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "i2c_candy.h"



static i2cc_msg_t i2cc_msg;
static uint8_t i2cc_bytes_idx;
static uint8_t  i2cc_msg_sync;

#define I2C_STATE_IDLE          0
#define I2C_STATE_START         1
static uint8_t i2c_state;
static uint8_t i2c_bit_count;
static uint8_t i2c_byte;



// called when SCK raising
ISR(INT0_vect) {
    if (i2c_bit_count >= 8) {
        i2c_bit_count = 0;
        if (i2cc_bytes_idx < sizeof(i2cc_msg_t)) {
            i2cc_msg.bytes[i2cc_bytes_idx] = i2c_byte;
        }
        i2cc_bytes_idx++;
    } else {
        i2c_byte <<= 1;
        if (PIND & I2C_SDA) i2c_byte |= 1;
        i2c_bit_count++;
    }
}


// called when SDA changes
ISR(INT1_vect) {
    // SDA changing matters only if SCL is high. Otherwise return immediately.
    if (PIND & I2C_SCL) {
        if ((PIND & I2C_SDA) && i2c_state == I2C_STATE_START) {
            // disable interrupt for SCL rising
            GICR &= ~_BV(INT0);
            i2c_bit_count = 0;
            i2c_state = I2C_STATE_IDLE;

            if (i2cc_bytes_idx == sizeof(i2cc_msg_t) &&
                        i2cc_msg.msg.hdr[0] == I2CC_HDR0 &&
                        i2cc_msg.msg.hdr[1] == I2CC_HDR1 &&
                        i2cc_msg.msg.hdr[2] == I2CC_HDR2) {
                uint8_t ccsum = 0;
                uint8_t i;

                // calculate checksum
                for (i = 0; i < sizeof(i2cc_msg_t) - 1; i++) {
                    ccsum += i2cc_msg.bytes[i];
                }

                if (i2cc_msg.msg.csum == ccsum) {
                    i2cc_msg_sync = 1;
                    i2cc_stop();
               }
           }

        } else if (i2c_state == I2C_STATE_IDLE) {    // START
            i2c_bit_count = 0;
            i2c_state = I2C_STATE_START;
            i2cc_bytes_idx = 0;

            // enable interrupt for SCL rising
            GICR |= _BV(INT0);
        }
    }
}



//
// interface functions
//

void i2cc_init() {
    // port D2, D3 - in, pull-up disabled
    DDRD &= ~(I2C_SCL | I2C_SDA);
    PORTD &= ~(I2C_SCL | I2C_SDA);

    // INT0 Interrupt - SCL interrupt on rising edge
    MCUCR |= _BV(ISC01) | _BV(ISC00);
    // INT1 Interrupt - SDA interrupt on any edge
    MCUCR |= _BV(ISC10);
}



void i2cc_start() {
    i2c_bit_count = 0;
    i2c_state = I2C_STATE_IDLE;
    i2cc_msg_sync = 0;

    // enable INT1 interrupts
    GICR |= _BV(INT1);
    GICR &= ~_BV(INT0);
}


void i2cc_stop() {
    GICR &= ~(_BV(INT1) | _BV(INT0));
}


i2cc_msg_t *i2cc_get_msg() {
    return i2cc_msg_sync == 0 ? NULL : &i2cc_msg;
}
